# This Code is part of the following publication:
# Schwarzmueller, F. & Kastner, T (2022), Agricultural trade and its impact on cropland use
# and the global loss of species' habitats. Sustainability Science, doi: 10.1007/s11625-022-01138-7
#
#
# Please cite ;-)
# (c) Florian Schwarzmller, December 2021

#### some initial things ####
#rm(list=ls())  #clears the work environment
rm(list= setdiff(ls(),c("directory", "conversion_opt","included_years", "prefer_import"))) # use this when run externally 
options(dplyr.summarise.inform=F)

### function to read in required packages
packages <- function(packagelist){
  for (package in packagelist) {
    if(!require(package, character.only = T)){
      install.packages(package, dependencies = T)
      require(package, character.only = T)
    }
  }
}

packagelist <- list("data.table", "tidyverse","readxl","igraph","pracma")
packages(packagelist)

#### choose options ####
# prefer import over export data ("import") or export over import ("export") 
if(!exists("prefer_import")){
  prefer_import="import"
}

# conversion method into primary items ("dry_matter","Energy","Protein","Fiber_TD","Zinc","Iron","Calcium",
#                                       "Folate_Tot","Riboflavin","Choline_Tot","Potassium","Vit_E","Vit_B12",
#                                       "Vit_K","Vit_A")
if(!exists("conversion_opt")){
  conversion_opt = "dry_matter"
}

if(!exists("included_years")){
  included_years = 2013
}

filename <-paste("results\\TradeMatrix_",prefer_import,"_",conversion_opt,"_",included_years,".csv",sep="")
log.filename <- paste("results\\logfile_TradeMatrix_",prefer_import,"_",conversion_opt,"_",included_years,".txt",sep="")
 

#### ####
#### functions ####

### eliminate years without reports ###

eliminate.dates <- function(fun.data){
  
  
  reporting.date.start <- select(reporting.date, Country_Code, Start_Year) %>%
    drop_na()
  
  reporting.date.end <- select(reporting.date, Country_Code, End_Year) %>%
    drop_na()
  
  j=1
  for (i in reporting.date.start$Country_Code){
    fun.data[which(fun.data$Reporter_Country_Code==i && fun.data$Year< reporting.date.start$Start_Year[j]),6] <-0
    j= j+1
  }
  
  j=1
  for (i in reporting.date.end$Country_Code){
    fun.data[which(fun.data$Reporter_Country_Code==i && fun.data$Year> reporting.date.end$End_Year[j]),6] <-0
  }
  
  
  return(fun.data)
}

primary.conversion <- function(conversion_map,conversion_option,display){
  index <- which(names(content.factors)==conversion_option)
  if (is_empty(index)){
    cat("Primary Conversion option not available")
    return()
  }
  joined <- left_join(conversion_map,select(content.factors,Item_Code,conversion_option),by=c("FAO_code"="Item_Code"))
  joined <- left_join(joined,select(content.factors,Item_Code,conversion_option),by=c("primary_item"="Item_Code"))
  joined$Conversion_factor <- joined[,5]/joined[,6]
  joined <- arrange(joined, FAO_code)
  
  #output
  if(display==T){
  cat("Division by zero when matching the following items: \n",file = log.filename, append = T)
  write.table(filter(joined,Conversion_factor==Inf) %>% select(FAO_code,FAO_name),file = log.filename, append = T, col.names = F, row.names = F)
  
  cat("No conversion values for the following items: \n",file = log.filename, append = T)
  write.table(filter(joined,is.na(Conversion_factor)) %>% select(FAO_code,FAO_name),file = log.filename, append = T, col.names = F, row.names = F)
  
  
  cat("Conversion factors set to 0 \n",file = log.filename, append = T)
  }
  
  joined[!is.finite(joined$Conversion_factor),7] <- 0
 
  return(filter(joined,!is.na(Conversion_factor)) %>% select(FAO_code,FAO_name,primary_item,FAO_name_primary,Conversion_factor))
}

matrix.operation <- function(item_Code, year){
  
  g <- primary.data %>%
    .[primary_item==item_Code &
        Year==year,c(1,2,5)] %>%
    graph_from_data_frame(directed=T) %>%
    get.adjacency(sparse=F,attr = "Value.Sum") 
  
  trade.data <- g[as.character(sort(as.numeric(rownames(g)))),as.character(sort(as.numeric(colnames(g))))]
  
  
  production <- production.all %>% 
    .[Item_Code==item_Code &
        Year==year &
        Element_Code==5510]
  
  if (nrow(production)==0){
    cat(paste("No production data for item code", item_Code,"in", year,"\n", sep=" "), file=log.filename, append=T)
    return(data.table("Consumer_Country_Code"=0,
                      "Producer_Country_Code"=0,
                      "Value"=0,
                      "Item_Code"=item_Code,
                      "Year"=year)
    )
  }
  
  prod <- data.frame(Area_Code = sort(as.numeric(rownames(g))))
  
  
  production_data <- full_join(prod, select(production, Area_Code,Value), by= "Area_Code")
  
  
  production_data[which(is.na(production_data[,2])),2]<-0
  
  missing <- anti_join(production_data, prod, by="Area_Code")
  
  for (m in missing[,1]){
    trade.data<-rbind(trade.data,0)
  }
  for (m in missing[,1]){
    trade.data<-cbind(trade.data,0)
  }
  
  ##### Problem is here #######
  
  rownames(trade.data)<-production_data[,1]
  colnames(trade.data)<-production_data[,1]
  ###replace with: 
  # imports2 <- rowSums(g)
  #imp <- data.table("Area_Code"=as.numeric(names(imports2)), "Import"=imports2)
  
  
  
  imports <- primary.data %>% 
    .[primary_item==item_Code & 
        Year==year] %>%
    group_by(Consumer_Country_Code)%>%
    summarise(sum(Value.Sum))
  
  names(imports) <- c("Area_Code", "Import")
  
  imports <- full_join(production_data, imports, by="Area_Code")
  
  imports[which(is.na(imports[,3])),3]<-0
  
  imports<- imports[,c(1,3)]
  
  exports <- primary.data %>% 
    .[primary_item==item_Code & 
        Year==year] %>%
    group_by(Producer_Country_Code)%>%
    summarise(sum(Value.Sum))
  
  names(exports) <- c("Area_Code", "Export")
  
  exports <- full_join(production_data, exports, by="Area_Code")
  
  exports[which(is.na(exports[,3])),3]<-0
  
  exports <- exports[,c(1,3)]
  
  x  <- production_data[,2] + imports[,2]
  
  ac <- x - exports[,2]
  
  one_over_x <-1/x
  
  one_over_x[which(one_over_x==Inf)]<-0
  
  A <- trade.data %*% diag(one_over_x)
  
  # if (det(diag(nrow(A))-A)==0){
  #   print(paste("Singular matrix for item code ", item_Code," in ", year, sep=""))
  #   return(c(0,0,0,0,0))
  # }
  
  R <- pinv(diag(nrow(A))-A) %*% diag(production_data[,2])
  
  c <- ac * one_over_x
  
  R_bar <- diag(c) %*% R
  
  colnames(R_bar)<-production_data[,1]
  
  b <- graph_from_adjacency_matrix(R_bar, mode= "directed", weighted=T)
  
  if (length(E(b))==0){
    print(paste("Return matrix == 0 for item code ", item_Code," in ", year, sep=""))
    return(data.table("Consumer_Country_Code"=0,
                      "Producer_Country_Code"=0,
                      "Value"=0,
                      "Item_Code"=item_Code,
                      "Year"=year)
    )
  }
  
  return.data <- as.data.frame(cbind( get.edgelist(b)))
  return.data$Value <- round(E(b)$weight,2) 
  
  names(return.data)<- c("Consumer_Country_Code", "Producer_Country_Code", "Value")
  
  return.data$Item_Code <- item_Code
  
  return.data$Year <- year
  
  return(return.data[return.data$Value!=0,])
}

#### ####
####Set Working Directory and load data ####

## either via a a dialog box
# dir <- choose.dir()

## or by setting the directory manually

if (!exists("directory")) { 
  #directory <- choose.dir()
  #directory <- ("C:\\Users\\fschwarzmueller\\Documents\\projekte\\einstieg\\FAO_Data\\V2\\")
  directory <- "C:\\Users\\Florian\\Documents\\Arbeit\\0520"
}
setwd(directory)

if(!dir.exists("results")) dir.create("results")

cat(paste("log file for run at",Sys.time(),"\n",sep=" "), file=log.filename)
cat("\n", file=log.filename, append=T)
cat(paste("Working directory",directory,"\n", sep=" "),file = log.filename, append = T)
cat("\n", file=log.filename, append=T)

### Trade data
trade.data.file <- "Trade_DetailedTradeMatrix_E_All_Data_(Normalized).csv"
raw.trade.data <- fread(trade.data.file)
names(raw.trade.data)<- str_replace_all(names(raw.trade.data),c(" "="_"))
raw.trade.data <- raw.trade.data[Year %in% included_years]
cat(paste("Trade data file:",trade.data.file,file.info(trade.data.file)$mtime,"\n"),file = log.filename, append = T)

### Reporting Data
reporting.data.file <- "Reporting_Dates.xls"
reporting.date <- read_xls(reporting.data.file,1)
names(reporting.date)<- str_replace_all(names(reporting.date),c(" "="_"))
cat(paste("Reporting dates file:",reporting.data.file,file.info(reporting.data.file)$mtime,"\n"),file = log.filename, append = T)

### Primary item data
item.map.file <- "primary_item_map_feed.csv"
item.map <- fread(item.map.file, sep=",", header = T)
names(item.map)<- str_replace_all(names(item.map),c(" "="_"))
cat(paste("Mapping to primary items using file:",item.map.file, file.info(item.map.file)$mtime,"\n"),file = log.filename, append = T)

##Conversion factors
content.factors.file <- "content_factors_per_100g.xlsx"
suppressMessages(
  content.factors <- read_excel(content.factors.file,col_names = T,skip=1)
)
names(content.factors)<- str_replace_all(names(content.factors),c(" "="_"))

suppressMessages(
units <- read_excel(content.factors.file,col_names = F,range= cell_rows(1))
)
names(units)<-names(content.factors)

cat(paste("Converting to primary items using file:",content.factors.file, file.info(content.factors.file)$mtime,"\n"),file = log.filename, append = T)

### Production data
production.crops.file <- "Production_Crops_E_All_Data_(Normalized).csv"
production.crops <- fread(production.crops.file)
names(production.crops)<- str_replace_all(names(production.crops),c(" "="_"))
production.crops <- production.crops[Year %in% included_years,1:11]
cat(paste("Production crops file:",production.crops.file, file.info(production.crops.file)$mtime,"\n"),file = log.filename, append = T)

production.animals.file <- "Production_LivestockPrimary_E_All_Data_(Normalized).csv"
production.animals <- fread(production.animals.file)
names(production.animals)<- str_replace_all(names(production.animals),c(" "="_"))
production.animals <- production.animals[Year %in% included_years,1:11]
cat(paste("Production livestock file:",production.animals.file, file.info(production.animals.file)$mtime,"\n"),file = log.filename, append = T)

production.offals.file <- "CommodityBalances_LivestockFish_E_All_Data_(Normalized).csv"
production.offals <- fread(production.offals.file)
names(production.offals) <- str_replace_all(names(production.offals),c(" "="_"))
production.offals <- production.offals[Year %in% included_years,1:11]
production.offals <- production.offals[production.offals$Element_Code==5510 & production.offals$Item_Code==2736,]
cat(paste("Production offals from file:",production.offals.file, file.info(production.offals.file)$mtime,"\n"),file = log.filename, append = T)


production.all <- rbind(production.crops,production.animals,fill=T)
production.all <- rbind(production.all, production.offals,fill=T)


production.all <- filter(production.all, Area_Code<300,Element_Code==5510)

rm(list= c("production.crops","production.animals", "production.offals"))

##
cat("\n", file=log.filename, append=T)
cat(paste("Conversion opinion:",conversion_opt,"in",select(units,conversion_opt),"\n"),file = log.filename, append = T)

### Data files for sugar conversion

sugar.processing.file <- "CommodityBalances_Crops_E_All_Data_(Normalized).csv"
sugar.processing <- fread(sugar.processing.file )
names(sugar.processing)<- str_replace_all(names(sugar.processing),c(" "="_"))
sugar.processing <- sugar.processing[Year %in% included_years]


#### harmonize import and export data ####

data.import <- select(raw.trade.data, Reporter_Country_Code, Partner_Country_Code, Element_Code, Item_Code, Year, Value) %>%
  filter(Element_Code ==5610)

data.export <- select(raw.trade.data,  Partner_Country_Code, Reporter_Country_Code, Element_Code, Item_Code, Year, Value) %>%
  filter(Element_Code ==5910)

rm(raw.trade.data)

data.import <- eliminate.dates(data.import) #eliminate Years without reports
data.export <- eliminate.dates(data.export) #

rm(reporting.date)

data.import[which(data.import[,1]==data.import[,2]),6] <- 0 #loops are set to zero
data.import <- filter(data.import,Value !=0)                #eliminate zeros
data.export[which(data.export[,1]==data.export[,2]),6] <-0  #loops are set to zero
data.export <- filter(data.export,Value !=0)                #eliminate zeros

if (prefer_import=="import") {
  cat("Prefering import over export data \n", file=log.filename,append=T)
  data.import <- rename(data.import,"Consumer_Country_Code"="Reporter_Country_Code","Producer_Country_Code"="Partner_Country_Code")
  data.export <- rename(data.export,"Consumer_Country_Code"="Partner_Country_Code","Producer_Country_Code"="Reporter_Country_Code")
  
  #combine import and export reports, import is prefered over export
  trade.data <- rbind(data.import,data.export) %>% 
    distinct(Consumer_Country_Code, Producer_Country_Code, Year, Item_Code,.keep_all=T)
} else if (prefer_import=="export"){
  cat("Prefering export over import data", file=log.filename,append=T)
  data.import <- rename(data.import,"Consumer_Country_Code"="Reporter_Country_Code","Producer_Country_Code"="Partner_Country_Code")
  data.export <- rename(data.export,"Consumer_Country_Code"="Partner_Country_Code","Producer_Country_Code"="Reporter_Country_Code")
  
  #combine import and export reports, import is prefered over export
  trade.data <- rbind(data.export,data.import) %>% 
    distinct(Consumer_Country_Code, Producer_Country_Code, Year, Item_Code,.keep_all=T)
} else {
  cat("No valid entry for treating import and export data", file=log.filename,append=T)
}

cat("\n", file=log.filename, append=T)

rm(list= c("data.import", "data.export"))

## sort data ##
trade.data <- arrange(trade.data, Consumer_Country_Code, Producer_Country_Code)


#### map to primary equivalents ####


trade.data <- left_join(trade.data, primary.conversion(item.map,conversion_opt,T), by=c("Item_Code"="FAO_code"))

trade.data <- mutate(trade.data, primary.Value= Value * Conversion_factor)

primary.data <- trade.data %>%
  group_by(Consumer_Country_Code, Producer_Country_Code, Year, primary_item) %>%
  summarise(Value.Sum = sum(primary.Value)) %>%
  ungroup() %>%
  filter(primary_item != 0, Value.Sum !="NaN")


rm(trade.data)

#### calculate sugar production and sugar production shares ####

sugar.crop.codes <- c(156,157)

sugar.production <- left_join(filter(production.all,Item_Code %in% sugar.crop.codes),primary.conversion(item.map,conversion_opt,F), by=c("Item_Code"="FAO_code"))
sugar.production$Value.new <- sugar.production$Value * sugar.production$Conversion_factor

# calculate shares for later 
sugar.shares <- sugar.production %>%
  group_by(Area_Code,Year) %>%
  mutate(share = Value/sum(Value,na.rm=T)) %>%
  ungroup() %>%
  select(Area_Code,Year,Item_Code,share) %>%
  filter(share>0)

# add production data

sugar.production <- sugar.production %>%
  group_by(Area_Code,Area,primary_item,FAO_name_primary,Element_Code,Element,Year_Code,Year,Unit) %>%
  summarise(Value = sum(Value.new,na.rm=T)) %>%
  ungroup() %>%
  filter(Value > 0) %>%
  rename(Item_Code = primary_item,Item = FAO_name_primary) %>%
  mutate(Flag=" ")


production.all <- rbind(production.all,sugar.production)

rm(list = c("sugar.production"))

#### calculate the trade matrix ####


q <- primary.data %>% distinct(Year, primary_item)


transformed.data <- data.frame("Consumer_Country_Code"=0,
                               "Producer_Country_Code"=0, 
                               "Value"=0,
                               "Item_Code"=0,
                               "Year"=0)

primary.data <- as.data.table(primary.data)
setkey(primary.data, "primary_item","Year")

production.all <- as.data.table(production.all)
setkey(production.all, "Item_Code","Year", "Element_Code")

cat("Calculating trade matrix \n", file=log.filename, append=T)


for (i in (1:nrow(q))){
  item_Code <- as.numeric(q[i,2])
  year <- as.numeric(q[i,1])
  #print(c(year,item_Code))
  transformed.data <- rbindlist(list(transformed.data, matrix.operation(item_Code,year)))
}
transformed.data <- filter(transformed.data, Consumer_Country_Code !=0)

transformed.data$Consumer_Country_Code <- as.numeric(as.character(transformed.data$Consumer_Country_Code))
transformed.data$Producer_Country_Code <- as.numeric(as.character(transformed.data$Producer_Country_Code))

#### identify primary items that are not traded ####
missing.prod.data <- production.all %>% 
  filter(Element_Code==5510,Year %in% included_years,Item_Code !="NA",Value!="NA") %>%
  filter(Item_Code %in% item.map$primary_item & !(Item_Code %in% unique(primary.data$primary_item)))

# and add them to the diagonal
add.data <- missing.prod.data %>%
  mutate(Consumer_Country_Code= Area_Code, Producer_Country_Code = Area_Code) %>%
  select(Consumer_Country_Code,Producer_Country_Code,Value,Item_Code,Year)

transformed.data <- rbind(transformed.data,add.data)

#### calculate and add sugar data ####

sugar.trade.data <- filter(transformed.data, Item_Code==2545)

sugar.processing <- filter(sugar.processing, Item_Code %in% c(2536,2537), Element_Code==5130, Area_Code<300, Value>0)
sugar.processing$Item_Code <- sugar.processing$Item_Code - 2380

sugar.processing <- left_join(sugar.processing,primary.conversion(item.map,"dry_matter",F), by=c("Item_Code"="FAO_code"))
sugar.processing$Value.new <- sugar.processing$Value*sugar.processing$Conversion_factor

sugar.processing <- sugar.processing %>%
  group_by(Area_Code,Year) %>%
  mutate(processing_share = Value.new/sum(Value.new)) %>%
  ungroup() %>%
  select(Area_Code,Year,Item_Code,processing_share)

sugar.crop.share <- left_join(sugar.shares,sugar.processing,by=c("Area_Code","Year","Item_Code"))


# check if a crop does not appear in processing data but does in production
sugar.crop.share <- sugar.crop.share %>%
  group_by(Area_Code,Year) %>%
  mutate(control=sum(processing_share,na.rm=T)) %>%
  ungroup()

# if that is the case, set the contribution to processing to zero
sugar.crop.share[which(is.na(sugar.crop.share$processing_share) & sugar.crop.share$control==1),5] <-0

# for all cases where there is no processing data, set the shares to production shares
sugar.crop.share[which(is.na(sugar.crop.share$processing_share)),5] <- sugar.crop.share[which(is.na(sugar.crop.share$processing_share)),4]

sugar.crop.share <- sugar.crop.share %>%
  rename(Sugar_Crop_Code=Item_Code) %>%
  mutate(Item_Code=2545)

sugar.crop.share <- left_join(sugar.crop.share,primary.conversion(item.map,"dry_matter",F),by=c("Sugar_Crop_Code"="FAO_code"))


# join with trade data 
sugar.data <- left_join(sugar.trade.data,sugar.crop.share,by=c("Year","Item_Code","Producer_Country_Code"="Area_Code"))
sugar.data <- mutate(sugar.data, Value.new=(Value*processing_share)/Conversion_factor)

sugar.data <- select(sugar.data,Consumer_Country_Code,Producer_Country_Code,Value=Value.new,Item_Code=Sugar_Crop_Code,Year)

# adjust domestic component (diagonal in the trade matrix)
sugar.production <- filter(production.all,Item_Code %in% sugar.crop.codes) %>% 
  rename(national_production=Value)

sugar.data <- sugar.data %>%
  group_by(Producer_Country_Code,Year,Item_Code) %>%
  mutate(sugar_crop_total=sum(Value))%>%
  ungroup()

sugar.data <- left_join(sugar.data,sugar.production, by=c("Producer_Country_Code"="Area_Code","Year","Item_Code"))
sugar.data$Value.new <- sugar.data$Value + sugar.data$national_production - sugar.data$sugar_crop_total
sugar.data[which(sugar.data$Producer_Country_Code==sugar.data$Consumer_Country_Code),]$Value <- sugar.data[which(sugar.data$Producer_Country_Code==sugar.data$Consumer_Country_Code),]$Value.new

sugar.data <- select(sugar.data,Consumer_Country_Code,Producer_Country_Code,Value,Item_Code,Year)

transformed.data <- rbind(filter(transformed.data,Item_Code != 2545),sugar.data)


#### write file ####
fwrite(transformed.data, filename)

