# This Code is part of the following publication:
# Schwarzmueller, F. & Kastner, T (2022), Agricultural trade and its impact on cropland use
# and the global loss of species' habitats. Sustainability Science, doi: 10.1007/s11625-022-01138-7
#
#
# Please cite ;-)
# (c) Florian Schwarzmller, December 2021

#### some initial things ####
#rm(list=ls())  #clears the work environment
rm(list= setdiff(ls(),c("directory","included_years", "conversion_opt","prefer_import"))) # use this when run externally 
options(dplyr.summarise.inform=F)

### function to read in required packages
packages <- function(packagelist){
  for (package in packagelist) {
    if(!require(package, character.only = T)){
      install.packages(package, dependencies = T)
      require(package, character.only = T)
    }
  }
}

packagelist <- list("data.table", "tidyverse","readxl","igraph","pracma")
packages(packagelist)

#### functions ####

ml.animal.prod <- function(y,country){
  production.animal.data.1 <- production.animals[production.animals$Year==y &
                                                   production.animals$Area_Code==country &
                                                   production.animals$Value >0,]
  
  if(nrow(production.animal.data.1)==0){
    cat(paste("No animal production data for country ", country, " in ", y,"\n",sep=""), file=log.filename, append = T)
    return(data.frame(Producer_Country_Code=country, Year=y, Animal_Product_Code=0, Item_Code=0, Value=0))
  }
  
  data.feed <- feed.data[feed.data$Year==y & 
                           feed.data$Area_Code==country &
                           feed.data$Value>0,]
  
  if(nrow(data.feed)==0){
    cat(paste("No feed production data for country ", country, " in ", y,"\n",sep=""), file=log.filename, append = T)
    return(data.frame(Producer_Country_Code=country, Year=y, Animal_Product_Code=0, Item_Code=0, Value=0))
  }
  
  data.2 <- inner_join(weighing.factors, production.animal.data.1, by= "Item_Code")
  data.2 <- filter(data.2,Weighing.factors>0)
  data.2 <- mutate(data.2,relative.weighing=(Weighing.factors/mean(Weighing.factors)))
  data.2 <- mutate(data.2,weighted.production=relative.weighing*Value)
  data.2 <- mutate(data.2,relative.production=weighted.production/sum(weighted.production))
  data.2 <- mutate(data.2,rel.prod.weighted=relative.production/Value)
  
  
  mpt <- outer(data.2$rel.prod.weighted,data.feed$Value)
  rownames(mpt) <- data.2$Item_Code
  colnames(mpt) <- data.feed$Primary_Item_Code
  
  return.data <- data.frame(Producer_Country_Code=country, Year=y, Animal_Product_Code=0, Item_Code=0, Value=0)
  
  
  for (j in seq(1:nrow(mpt))){
    tmp <- data.frame(Producer_Country_Code=country, 
                      Year=y, 
                      Animal_Product_Code=rownames(mpt)[j], 
                      Item_Code= colnames(mpt),
                      Value=mpt[j,])
    return.data <- rbind(return.data,tmp)
  }
  
  return.data <- return.data[-1,]
  return.data <-return.data[return.data$Value>0 & !is.na(return.data$Animal_Product_Code),]
  
  return(return.data)
}


primary.conversion <- function(conversion_map,conversion_option){
  index <- which(names(content.factors.CB)==conversion_option)
  if (is_empty(index)){
    cat("Primary Conversion option not available", file=log.filename, append=T)
    return()
  }
  joined <- left_join(conversion_map,select(content.factors.CB,Item_Code,conversion_option),by="Item_Code")
  joined <- left_join(joined,select(content.factors.CB,Item_Code,conversion_option),by=c("Primary_Item_Code"="Item_Code"))
  joined$Conversion_factor <- joined[,3]/joined[,4]
  joined <- arrange(joined, Primary_Item_Code)
  
  #output
  cat("No conversion possible using this method for the following items: \n",file = log.filename, append = T)
  write.table(filter(joined,is.na(Conversion_factor)) %>% select(Item_Code),file = log.filename, append = T, col.names = F, row.names = F)
  
  return(filter(joined,!is.na(Conversion_factor)) %>% select(Item_Code,Primary_Item_Code,Conversion_factor))
}


#### set directory ####

## setting the directory manually
if (!exists("directory")) { 
  directory <- "C:\\Users\\Florian\\Documents\\Arbeit\\0520"
}
setwd(directory)

#### choose options ####
if(!exists("included_years")){
  included_years = 2013
}

if(!exists("prefer_import")){
  prefer_import = "import"
}
# conversion method into primary items ("dry_matter","Energy","Protein","Fiber_TD","Zinc","Iron","Calcium",
#                                       "Folate_Tot","Riboflavin","Choline_Tot","Potassium","Vit_E","Vit_B12",
#                                       "Vit_K","Vit_A")
if(!exists("conversion_opt")){
  conversion_opt = "dry_matter"
}

filename <-paste("results\\TradeMatrixFeed_",prefer_import,"_",conversion_opt,"_",included_years,".csv",sep="")
log.filename <- paste("results\\logfile_TradeMatrixFeed_",prefer_import,"_",conversion_opt,"_",included_years,".txt",sep="")


cat(paste("log file for run at",Sys.time(),"\n",sep=" "), file=log.filename)
cat("\n", file=log.filename, append=T)
cat(paste("Working directory",directory,"\n", sep=" "),file = log.filename, append = T)

#### read in data ####

# mapping files
CB.map.file<- "CB_to_primary_items_map.csv"
CB.map <- read.csv(CB.map.file)

CB.split.file <- "CB_items_split.csv"
CB.split <- read.csv(CB.split.file)

cat(paste("Mapping items using files: \n",
          CB.map.file,file.info(CB.map.file)$mtime,"\n",
          CB.split.file, file.info(CB.split.file)$mtime, "\n"),file = log.filename, append = T)


# conversion factors
content.factors.file <- "content_factors_per_100g.xlsx"
suppressMessages(
  content.factors <- read_excel(content.factors.file,col_names = T,skip=1)
)
names(content.factors)<- str_replace_all(names(content.factors),c(" "="_"))

suppressMessages(
  units <- read_excel(content.factors.file,col_names = F,range= cell_rows(1))
)
names(units)<-names(content.factors)

cat(paste("Converting to primary items using file:",content.factors.file, file.info(content.factors.file)$mtime,"\n"),file = log.filename, append = T)
cat(paste("Conversion opinion:",conversion_opt,"in",select(units,conversion_opt),"\n"),file = log.filename, append = T)

CB.conversion.map.file <- "CB_code_FAO_code_for_conversion_factors.csv"
CB.conversion.map <- fread(CB.conversion.map.file)

content.factors.CB <- left_join(CB.conversion.map,content.factors, by=c("FAO_code"="Item_Code")) %>% select(-c(Item, FAO_code, FAO_name)) %>% rename(Item_Code = CB_code, Item=CB_name)
rm(list=c("content.factors","CB.conversion.map"))

#

# Feed use data
CB.crops.data.file <- "CommodityBalances_Crops_E_All_Data_(Normalized).csv"
CB.crops.data <- fread(CB.crops.data.file)
names(CB.crops.data)<- str_replace_all(names(CB.crops.data),c(" "="_"))
CB.crops.data <- filter(CB.crops.data,Year==included_years, Element_Code==5520)

# trade matrix
transformed.data.file <- paste("results\\TradeMatrix_",prefer_import,"_",conversion_opt,"_",included_years,".csv",sep="")
if(!file.exists(transformed.data.file)){ 
  transformed.data.file <- choose.files(default = paste("TradeMatrix_",prefer_import,"_",conversion_opt,"_",included_years, sep=""),
                                        caption= "Unable to find the corresponding trade matrix file. Please select.", multi=F)
}
transformed.data <- fread(transformed.data.file)

# Production data
production.animals.file <- "Production_LivestockPrimary_E_All_Data_(Normalized).csv"
production.animals <- fread(production.animals.file)
names(production.animals)<- str_replace_all(names(production.animals),c(" "="_"))
production.animals <- filter(production.animals,Year==included_years,Element_Code==5510)

# weighing factors for feed conversion
weighing.factors.file <- "weighing_factors.csv"
weighing.factors <- read.table(weighing.factors.file, sep=",", header = T)

#### data preperation ####
## join animal CB elements according to CB.map ##

CB.data <- left_join(CB.crops.data, primary.conversion(CB.map,conversion_opt), by=c("Item_Code"))

rm(CB.crops.data)

CB.data$Value.new <- CB.data$Value*CB.data$Conversion_factor

feed.data <- CB.data %>% filter(Area_Code <300, Primary_Item_Code!= "NA") %>%
  group_by(Area_Code,Primary_Item_Code,Year) %>%
  summarise(Value = sum(Value.new)) %>%
  ungroup()


#### calculate feed equivalents per ton of animal. product ####

d <- distinct(transformed.data,Year,Producer_Country_Code, .keep_all = F) %>% arrange(Year,Producer_Country_Code)

feed.eq.data <- data.frame(Producer_Country_Code=0, 
                            Year=0, 
                            Animal_Product_Code=0, 
                            Item_Code=0, Value=0)

production.animals <- as.data.table(production.animals[production.animals$Element_Code==5510,])
setkey(production.animals, "Year","Area_Code")

feed.data <- as.data.table(feed.data)
setkey(feed.data, "Year", "Area_Code")


for(q in seq(1:nrow(d))){
y <- d$Year[q]
country <- d$Producer_Country_Code[q]
feed.eq.data <- rbindlist(list(feed.eq.data, ml.animal.prod(y,country)))

}
no.CB.data <- feed.eq.data[Value==0]
feed.eq.data <- feed.eq.data[Value!=0]
feed.eq.data$Animal_Product_Code <- as.numeric(feed.eq.data$Animal_Product_Code)
feed.eq.data$Item_Code <- as.numeric(feed.eq.data$Item_Code)
feed.eq.data <- rename(feed.eq.data,"AP_Producer_Country_Code"="Producer_Country_Code","Feed_per_AP"="Value")
rm(list=c("production.animals","feed.data", "weighing.factors", "CB.map","CB.data","content.factors.CB"))

# combine CB_split with transformed.data to calculate the shares of traded items per feed item

crop.shares <- left_join(transformed.data,CB.split, by=c("Item_Code"="Primary_Item_Code")) %>%
  filter(Item_Code<867) %>%
  group_by(Consumer_Country_Code,CB_Item_Code,Year) %>%
  mutate(share=Value/sum(Value))%>%
  ungroup() %>%
  rename("Feed_Producer_Country_Code"="Producer_Country_Code","Feed_Item_Code"="Item_Code")

crop.shares[crop.shares$Value==0,]$share <-0
share.per.country <- left_join(feed.eq.data,crop.shares, by=c("AP_Producer_Country_Code"="Consumer_Country_Code","Year","Item_Code"="CB_Item_Code"))
share.per.country$feed_share <- share.per.country$Feed_per_AP * share.per.country$share
no.import.data <- share.per.country[which(is.na(share.per.country$feed_share)),]
rm(list=c("crop.shares","feed.eq.data"))
gc()

#animal.trade.data <- filter(transformed.data,Item_Code>850)
animal.trade.data <- as.data.table(filter(transformed.data,Item_Code>850))
share.per.country <- as.data.table(select(share.per.country,AP_Producer_Country_Code,Year,Animal_Product_Code,Year,Feed_Producer_Country_Code,Feed_Item_Code,feed_share))

animal.product.data.full <- merge(animal.trade.data,share.per.country,
                                  by.x = c("Year","Item_Code","Producer_Country_Code"),
                                  by.y = c("Year","Animal_Product_Code","AP_Producer_Country_Code"),
                                  allow.cartesian = T)

animal.product.data.full[,tons_feed_use:=Value*feed_share]
#animal.product.data.full[,c("feed_share","Producer_Country_Code","Value"):=NULL]
rm(list=c("animal.trade.data","share.per.country"))
gc()

for (i in unique(animal.product.data.full$Item_Code)){
  animal.product.data.sub <- animal.product.data.full[Item_Code==i]
  feed.in.animal.products.sub <- animal.product.data.sub[,.(tons_feed_use=sum(tons_feed_use)),
                                                    by = c("Year","Feed_Producer_Country_Code","Consumer_Country_Code","Feed_Item_Code")]

  feed.in.animal.products.sub <- rename(feed.in.animal.products.sub,Producer_Country_Code=Feed_Producer_Country_Code, Value=tons_feed_use,Item_Code=Feed_Item_Code)
  feed.in.animal.products.sub$Animal_Product_Code <- i

  if (!exists("feed.in.animal.products")) {
    feed.in.animal.products <- feed.in.animal.products.sub
  } else {
    feed.in.animal.products <- rbind(feed.in.animal.products,feed.in.animal.products.sub)
  }
  rm(feed.in.animal.products.sub)
  gc()
}
#### substract new links from original trade matrix  ####

#for all items, per country link (e.g. all 15 going from 1 to 2)
for (i in unique(animal.product.data.full$Item_Code)){
  animal.product.data.sub <- animal.product.data.full[Item_Code==i]
  feed.use.origin.per.country.sub <- animal.product.data.sub[,.(tons_feed_use=sum(tons_feed_use)),
                                                        by = c("Year","Feed_Producer_Country_Code","Producer_Country_Code","Feed_Item_Code")]
  if (!exists("feed.use.origin.per.country")){
    feed.use.origin.per.country <- feed.use.origin.per.country.sub
  } else {
    feed.use.origin.per.country <- rbind(feed.use.origin.per.country,feed.use.origin.per.country.sub)
  }
  rm(feed.use.origin.per.country.sub)
  gc()
}
  #multiply by -1
feed.use.origin.per.country$tons_feed_use <- feed.use.origin.per.country$tons_feed_use * (-1)
#rename to fit the trade data nomenclature
feed.use.origin.per.country <- rename(feed.use.origin.per.country, Value=tons_feed_use,Item_Code=Feed_Item_Code, Producer_Country_Code=Feed_Producer_Country_Code, Consumer_Country_Code=Producer_Country_Code)
gc()
# trade data of non-animal products
crop.trade.data <- as.data.table(filter(transformed.data,Item_Code<850))

# merge with above and substract
crop.trade.data <-rbind(crop.trade.data,feed.use.origin.per.country)
crop.trade.data <- crop.trade.data[,.(Value=sum(Value)),
                                   by=c("Year", "Producer_Country_Code", "Consumer_Country_Code","Item_Code")]

#add empty column
crop.trade.data$Animal_Product_Code <- NA

#combine data and write file
fwrite(rbind(crop.trade.data,feed.in.animal.products), filename)

error.filename <- unlist(strsplit(filename,"[.]"))[1]
fwrite(no.CB.data[,1],paste(error.filename,"_nofeeddata.csv",sep=""))
fwrite(no.import.data[,1:4],paste(error.filename,"_noimportdata.csv",sep=""))


